/// <reference types="node" />
import { Buffer } from 'buffer';
import { TxMsg } from './tx-msg';
/**
 * Model representing a MsgAppStake to stake as an Application in the Pocket Network
 */
export declare class MsgProtoAppStake extends TxMsg {
    readonly KEY: string;
    readonly AMINO_KEY: string;
    readonly pubKey: Buffer;
    readonly chains: string[];
    readonly amount: string;
    /**
     * Constructor for this class
     * @param {Buffer} pubKey - Public key buffer
     * @param {string[]} chains - Network identifier list to be requested by this app
     * @param {string} amount - The amount to stake, must be greater than 0
     */
    constructor(pubKey: string, chains: string[], amount: string);
    /**
     * Returns the msg type key
     * @returns {string} - Msg type key value.
     * @memberof MsgAppStake
     */
    getMsgTypeKey(): string;
    /**
     * Converts an Msg Object to StdSignDoc
     * @returns {object} - Msg type key value.
     * @memberof MsgAppStake
     */
    toStdSignDocMsgObj(): object;
    /**
     * Converts an Msg Object for StdTx encoding
     * @returns {any} - Msg type key value.
     * @memberof MsgAppStake
     */
    toStdTxMsgObj(): any;
}
//# sourceMappingURL=msg-proto-app-stake.d.ts.map