/// <reference types="node" />
import { Buffer } from 'buffer';
import { TxMsg } from './tx-msg';
/**
 * Model representing a MsgNodeStake to stake as an Node in the Pocket Network
 */
export declare class MsgProtoNodeStakeTx extends TxMsg {
    readonly KEY: string;
    readonly AMINO_KEY: string;
    readonly DEFAULT_PORT: string;
    readonly DEFAULT_PROTOCOL: string;
    readonly pubKey: Buffer;
    readonly outputAddress: Buffer;
    readonly chains: string[];
    readonly amount: string;
    readonly serviceURL: URL;
    readonly rewardDelegators: {
        [key: string]: number;
    } | undefined;
    /**
     * @param {string} pubKey - Public key
     * @param {string[]} chains - String array containing a list of blockchain hashes
     * @param {string} amount - Amount to be sent, has to be a valid number and cannot be lesser than 0
     * @param {URL} serviceURL - Service node URL, needs to be https://
     * @param {Object.<string, number>} rewardDelegators - Reward delegators
     */
    constructor(pubKey: string, outputAddress: string, chains: string[], amount: string, serviceURL: URL, rewardDelegators: {
        [key: string]: number;
    } | undefined);
    /**
     * Returns the parsed serviceURL
     * @returns {string} - Parsed serviceURL
     * @memberof MsgNodeStake
     */
    private getParsedServiceURL;
    /**
     * Converts an Msg Object to StdSignDoc
     * @returns {object} - Msg type key value.
     * @memberof MsgNodeStake
     */
    toStdSignDocMsgObj(): object;
    /**
     * Converts an Msg Object to StdSignDoc
     * @returns {any} - Msg type key value.
     * @memberof MsgNodeStake
     */
    toStdTxMsgObj(): any;
}
//# sourceMappingURL=msg-proto-node-stake.d.ts.map